@@ -22,6 +22,7 @@
 #include <string.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-macros.h>
+#include <libgnome/gnome-program.h>
 #include <gconf/gconf-client.h>
 
 #include "headers.h"
@@ -39,13 +40,13 @@ typedef struct {
 } SplashApp;
 
 static const SplashApp splash_map_table[] = {
-	{ N_("Sawfish Window Manager"),  "sawfish",               "gnome-ccwindowmanager.png" },
-	{ N_("Metacity Window Manager"), "metacity",              "gnome-ccwindowmanager.png" },
-	{ N_("Window Manager"),          "gnome-wm",              "gnome-ccwindowmanager.png" },
-	{ N_("The Panel"),               "gnome-panel",           "gnome-panel.png" },
-	{ N_("Session Manager Proxy"),   "gnome-smproxy",         "gnome-session.png" },
-	{ N_("Nautilus"),                "nautilus",              "gnome-ccdesktop.png" },
-	{ N_("Desktop Settings"),        "gnome-settings-daemon", "gnome-settings.png" }
+	{ N_("Sawfish Window Manager"),  "sawfish",               "gnome-window-manager" },
+	{ N_("Metacity Window Manager"), "metacity",              "gnome-window-manager" },
+	{ N_("Window Manager"),          "gnome-wm",              "gnome-window-manager" },
+	{ N_("The Panel"),               "gnome-panel",           "gnome-panel" },
+	{ N_("Session Manager Proxy"),   "gnome-smproxy",         "gnome-session" },
+	{ N_("Nautilus"),                "nautilus",              "gnome-fs-desktop" },
+	{ N_("Desktop Settings"),        "gnome-settings-daemon", "gnome-settings" }
 };
 
 static const SplashApp *
@@ -303,26 +304,57 @@ splash_widget_class_init (SplashWidgetClass *klass)
 	widget_class->button_release_event = splash_widget_button_release_event;
 }
 
-static void
-read_background (SplashWidget *sw)
+static GdkPixbuf *
+load_background_pixbuf (const char *filename)
 {
-	char *filename;
-	GdkPixbuf *pb;
-	GConfClient * client;
+	GdkPixbuf *retval;
+	GError    *error = NULL;
+	char      *path;
+
+	if (!filename)
+		return NULL;
+
+	path = gnome_program_locate_file (NULL,
+					  GNOME_FILE_DOMAIN_PIXMAP,
+					  filename,
+					  TRUE,
+					  NULL);
+	if (!path)
+		return NULL;
+
+	retval = gdk_pixbuf_new_from_file (path, &error);
+	if (!retval) {
+		g_warning ("Failed to load image from '%s': %s\n",
+			   path, error->message);
+		g_free (path);
+		g_error_free (error);
+		return NULL;
+	}
+
+	g_free (path);
+
+	return retval;
+}
+
+static void
+load_background (SplashWidget *sw)
+{
+	GConfClient *client;
+	char        *filename;
 
 	client = gconf_client_get_default ();
 
 	filename = gconf_client_get_string (client,
 					    "/apps/gnome-session/options/splash_image",
 					    NULL);
-	if (!filename)
-		filename = g_strdup (GNOME_ICONDIR "/splash/gnome-splash.png");
 
-	pb = gdk_pixbuf_new_from_file (filename, NULL);
+	g_object_unref (client);
+
+	sw->background = load_background_pixbuf (filename);
 	g_free (filename);
 
-	sw->background = pb;
-	g_object_unref (G_OBJECT (client));
+	if (!sw->background)
+		sw->background = load_background_pixbuf ("splash/gnome-splash.png");
 }
 
 static void
@@ -356,7 +388,7 @@ splash_widget_instance_init (SplashWidget *sw)
 
 	sw->icon_theme = gnome_icon_theme_new ();
         
-	read_background (sw);
+	load_background (sw);
 }
 
 static GdkPixbuf *
@@ -465,59 +497,50 @@ void
 splash_widget_add_icon (SplashWidget *sw,
 			const char   *executable_name)
 {
-	char *text;
-	char *basename;
-	GdkPixbuf *pb;
 	const SplashApp *app;
+	char            *basename;
+	GdkPixbuf       *pb = NULL;
 
 	g_return_if_fail (SPLASH_IS_WIDGET (sw));
 
 	if (!executable_name || executable_name [0] == '\0')
 		return;
 
-	basename = g_path_get_basename (executable_name);
-
-	pb = NULL;
-	text = NULL;
-	app = get_splash_app (basename);
-
-	if (app) {
-		pb = get_splash_icon (sw, app->icon);
-		text = _(app->human_name);
-	}
-
 	/* FIXME: we should allow arbitrary apps to install
          * stuff here, by passing the icon name to the SM
          * as a session property
          */
 
+	basename = g_path_get_basename (executable_name);
+	app = get_splash_app (basename);
+
+	if (app)
+		pb = get_splash_icon (sw, app->icon);
+
 	if (!pb)
 		pb = get_splash_icon (sw, basename);
 
-	if (!pb)
-		pb = get_splash_icon (sw, "gnome-unknown.png");
-
-	if (!text)
-		text = basename;
-
-	/* re-draw the old text extents */
-	gtk_widget_queue_draw_area (
-		GTK_WIDGET (sw),
-		sw->text_box.x, sw->text_box.y,
-		sw->text_box.width, sw->text_box.height);
-
-	pango_layout_set_text (sw->layout, text, -1);
-	calc_text_box (sw);
-
-	/* re-draw the new text extents */
-	gtk_widget_queue_draw_area (
-		GTK_WIDGET (sw),
-		sw->text_box.x, sw->text_box.y,
-		sw->text_box.width, sw->text_box.height);
-
 	if (pb) {
-		SplashIcon *si;
-		GdkRectangle area;
+		SplashIcon   *si;
+		GdkRectangle  area;
+		char         *text;
+
+		text = app && app->human_name ?  _(app->human_name) : basename;
+
+		/* re-draw the old text extents */
+		gtk_widget_queue_draw_area (
+			GTK_WIDGET (sw),
+			sw->text_box.x, sw->text_box.y,
+			sw->text_box.width, sw->text_box.height);
+
+		pango_layout_set_text (sw->layout, text, -1);
+		calc_text_box (sw);
+
+		/* re-draw the new text extents */
+		gtk_widget_queue_draw_area (
+			GTK_WIDGET (sw),
+			sw->text_box.x, sw->text_box.y,
+			sw->text_box.width, sw->text_box.height);
 
 		si = g_new0 (SplashIcon, 1);
 		si->unscaled = pb;
@@ -531,6 +554,8 @@ splash_widget_add_icon (SplashWidget *sw,
 			area.x, area.y,
 			area.width, area.height);
 	}
+
+	g_free (basename);
 }
 
 static SplashWidget *global_splash = NULL;
